﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace ELTE.Windows.ThesisGenerator.Model
{
    /// <summary>
    /// Tételgenerátor modell típusa.
    /// </summary>
    public class ThesisGeneratorModel
    {
        private Int32 _ThesisCount;
        private Int32 _PeriodCount;
        private List<Int32> _HistoryList;
        private Random _ThesisGenerator;

        /// <summary>
        /// Tételek számának lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 ThesisCount
        {
            get { return _ThesisCount; }
            set 
            {
                if (value <= 0)
                    throw new InvalidOperationException("Thesis count must be a positive value.");
                _ThesisCount = value;
                if (_PeriodCount >= value) // ellenőrizzük a periódushosszt
                {
                    _PeriodCount = value - 1;
                    while (_HistoryList.Count > _PeriodCount) // a történeti lista hosszát 
                        _HistoryList.RemoveAt(0);
                }
                for (Int32 i = History.Count - 1; i >= 0; i--) // ellenőrizzük a történeti elemeket is
                    if (_HistoryList[i] > _ThesisCount)
                        _HistoryList.RemoveAt(i);
            }
        }
        /// <summary>
        /// Periódushossz lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 PeriodCount 
        { 
            get { return _PeriodCount; }
            set 
            {
                if (value >= _ThesisCount)
                    throw new InvalidOperationException("Period count must be smaller than the thesis count.");
                if (value < 0)
                    throw new InvalidOperationException("Period count cannot be negative.");

                _PeriodCount = value;
                while (_HistoryList.Count > _PeriodCount) // ellenőrizzük a történeti lista hosszát 
                    _HistoryList.RemoveAt(0);
            }
        }
        /// <summary>
        /// Történet lekérdezése.
        /// </summary>
        public ReadOnlyCollection<Int32> History { get; private set; }

        /// <summary>
        /// Tételgenerátor modell példányosítása.
        /// </summary>
        /// <param name="thesisCount">Tételek száma.</param>
        /// <param name="periodCount">Periódushossz.</param>
        public ThesisGeneratorModel(Int32 thesisCount, Int32 periodCount)
        {
            if (thesisCount <= 0)
                throw new ArgumentException("Thesis count must be a positive value.", "thesisCount");
            if (periodCount >= thesisCount)
                throw new ArgumentException("Period count must be smaller than the thesis count.", "periodCount");
            if (periodCount < 0)
                throw new ArgumentException("Period count cannot be negative.", "periodCount");

            _ThesisCount = thesisCount;
            _PeriodCount = periodCount;

            _HistoryList = new List<Int32>();
            History = new ReadOnlyCollection<Int32>(_HistoryList);

            _ThesisGenerator = new Random();
        }
        /// <summary>
        /// Új tételszám generálása.
        /// </summary>
        /// <returns>A generált tételszám.</returns>
        public Int32 Generate()
        {
            Int32 number = _ThesisGenerator.Next(1, _ThesisCount + 1); // új szám generálása 1 és a tételszám között
            while (History.Contains(number)) // ha a szám szerepel a korábbiak között
                number = _ThesisGenerator.Next(1, _ThesisCount + 1); // akkor új generálása

            return number;
        }
        /// <summary>
        /// Tétel elfogadása.
        /// </summary>
        /// <param name="number">A tétel sorszáma.</param>
        public void Take(Int32 number)
        {
            if (number <= 0 || number > _ThesisCount)
                throw new ArgumentException("The argument is not a thesis number", "number");
            if (History.Contains(number))
                throw new ArgumentException("The argument is already in the history", "number");

            _HistoryList.Add(number); // felvesszük a számok közé
            if (_HistoryList.Count > _PeriodCount) // ha túlcsordulás történt, töröljük a legrégebbi tételt
                _HistoryList.RemoveAt(0);
        }
        /// <summary>
        /// Tételt törlése a történetből.
        /// </summary>
        /// <param name="number">A tétel sorszáma.</param>
        public void Drop(Int32 number)
        {
            if (number <= 0 || number > _ThesisCount)
                throw new ArgumentException("The argument is not a thesis number", "number");
            if (!_HistoryList.Contains(number))
                throw new ArgumentException("The argument is not in the history", "number");

            _HistoryList.Remove(number);
        }
    }
}
