﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Threading;
using ELTE.Windows.ThesisGenerator.Model;

namespace ELTE.Windows.ThesisGenerator.ViewModel
{
    /// <summary>
    /// Tételgenerátor nézetmodell típusa.
    /// </summary>
    public class ThesisGeneratorViewModel : INotifyPropertyChanged
    {
        private ThesisGeneratorModel _Model;
        private DispatcherTimer _Timer;
        private Int32 _CurrentThesis;
        private Boolean _IsSet;

        /// <summary>
        /// Felirat lekérdezése.
        /// </summary>
        public String Text { get; private set; }
        /// <summary>
        /// Aktuális tétel lekérdezése.
        /// </summary>
        public String CurrentThesis { get { return _CurrentThesis.ToString(); } }
        /// <summary>
        /// Tételek számának lekérdezése, vagy beállítása.
        /// </summary>
        public String ThesisCount { get; set; }
        /// <summary>
        /// Periódushossz lekérdezése, vagy beállítása.
        /// </summary>
        public String PeriodCount { get; set; }

        /// <summary>
        /// Történet lekérdezése.
        /// </summary>
        public ObservableCollection<HistoryItem> History { get; private set; }

        public DelegateCommand StartStopCommand { get; private set; }
        public DelegateCommand OpenSettingsCommand { get; private set; }
        public DelegateCommand SaveSettingsCommand { get; private set; }

        /// <summary>
        /// Tulajdonság változásának eseménye.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// Üzenetjelzés eseménye.
        /// </summary>
        public event EventHandler<ApplicationMessageEventArgs> ApplicationMessaged;
        /// <summary>
        /// Beállítások megnyitásának eseménye.
        /// </summary>
        public event EventHandler OpenSettings;
        /// <summary>
        /// Beállítások bezárásának eseménye.
        /// </summary>
        public event EventHandler SaveSettings;

        public ThesisGeneratorViewModel()
        {
            _Model = new ThesisGeneratorModel(1, 0); // modell létrehozása alapértekékkel
            _CurrentThesis = 1;
            _IsSet = false;

            _Timer = new DispatcherTimer(); // időzítő
            _Timer.Interval = TimeSpan.FromMilliseconds(50);
            _Timer.Tick += new EventHandler(Timer_Tick);

            Text = "Indít";
            ThesisCount = _Model.ThesisCount.ToString();
            PeriodCount = _Model.PeriodCount.ToString();
            History = new ObservableCollection<HistoryItem>();
            GenerateHistory();

            StartStopCommand = new DelegateCommand(
                param => _IsSet,            // tevékenység végrehajtásának feltétele
                param => TimerStartStop()   // tevékenység
            );
            OpenSettingsCommand = new DelegateCommand(param =>
            {
                ThesisCount = _Model.ThesisCount.ToString(); // beállítjuk az értékeket
                PeriodCount = _Model.PeriodCount.ToString();
                OnOpenSettings();
            });
            SaveSettingsCommand = new DelegateCommand(param => 
            {
                try // megpróbáljuk elmenteni az értékeket
                {
                    _Model.ThesisCount = Int32.Parse(ThesisCount);
                }
                catch
                {
                    OnApplicationMessaged("Az érték nem megfelelő.", MessageType.Error);
                    return;
                }
                try
                {
                    _Model.PeriodCount = Int32.Parse(PeriodCount);
                }
                catch
                {
                    OnApplicationMessaged("Az érték nem megfelelő.", MessageType.Error);
                    return;
                }
                OnPropertyChanged("ThesisCount");
                OnPropertyChanged("PeriodCount");
                _IsSet = true; // megtörtént a beállítás
                StartStopCommand.OnCanExecuteChanged(); // megváltozott a tevékenység végrehajthatósága
                OnSaveSettings();
                GenerateHistory();
            });
        }

        /// <summary>
        /// Történeti lista generálása.
        /// </summary>
        private void GenerateHistory()
        {
            History.Clear();
            for (Int32 i = 1; i <= _Model.ThesisCount; i++)
            { 
                History.Add(new HistoryItem(_Model, i)); // létrehzozzuk a lista elemeit
            }
            OnPropertyChanged("History");
        }

        /// <summary>
        /// Időzítő indítása/megállítása.
        /// </summary>
        private void TimerStartStop()
        {
            if (!_IsSet)
            {
                OnApplicationMessaged("Előbb állítsd be a paramétereket!", MessageType.Information);
                return;
            }

            if (_Timer.IsEnabled) // ha már fut az időzítő
            {
                _Timer.Stop(); // leállítjuk
                _Model.Take(_CurrentThesis); // elfogadjuk az aktuális tételt
                GenerateHistory();
                Text = "Indít";
                OnPropertyChanged("Text");
            }
            else 
            {
                Text = "Leállít";
                OnPropertyChanged("Text");
                _Timer.Start(); // ha nem fut, elindítjuk
            }
        }

        /// <summary>
        /// Időzítő eseménykezelője.
        /// </summary>
        /// <param name="sender">Küldő.</param>
        /// <param name="e">Eseményargumetum.</param>
        private void Timer_Tick(object sender, EventArgs e)
        {
            _CurrentThesis = _Model.Generate();
            OnPropertyChanged("CurrentThesis");
        }

        /// <summary>
        /// Tulajdonság változása ellenőrzéssel.
        /// </summary>
        /// <param name="propertyName">Tulajdonság neve.</param>
        private void OnPropertyChanged(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Alkalmazás üzenet eseménykiváltása.
        /// </summary>
        /// <param name="message">Üzenet szövege.</param>
        /// <param name="type">Üzenet típusa.</param>
        private void OnApplicationMessaged(String message, MessageType type)
        {
            if (ApplicationMessaged != null)
                ApplicationMessaged(this, new ApplicationMessageEventArgs { Message = message, Type = type });
        }

        /// <summary>
        /// Beállítások megnyitásának eseménykiváltása.
        /// </summary>
        private void OnOpenSettings()
        {
            if (OpenSettings != null)
                OpenSettings(this, EventArgs.Empty);
        }

        /// <summary>
        /// Beállítások bezárásának eseménykiváltása.
        /// </summary>
        private void OnSaveSettings()
        {
            if (SaveSettings != null)
                SaveSettings(this, EventArgs.Empty);
        }
    }
}
